#include "installenvpage.h"
#include <QVBoxLayout>
#include <QLabel>
#include <QProcess>
#include <QDebug>

#include <component/keywordhighlighter.h>
#include <component/outputedit.h>
#include <logic/KMRef.h>
#include <logic/KMBuilderSearch.h>
#include "messageboxutils.h"
#include "common/subprocess.h"
class InstallEnvPagePrivate : public QObject
{
    Q_OBJECT

public:
    InstallEnvPagePrivate(InstallEnvPage * parent);
    virtual ~InstallEnvPagePrivate();

    void setupUi();

private:
    Q_DECLARE_PUBLIC(InstallEnvPage)
    InstallEnvPage * q_ptr;

//    QProgressBar *m_progressBar;
//    int m_progress = 0;
//    QTextEdit *m_logText;

    OutputEdit * m_output;
    KeywordHighlighter * m_highlighter = nullptr;
    SubProcess m_subProcess;
    bool m_installationSuccess;
    QString m_baseRef;
    QString m_runtimeRef;
};


InstallEnvPage::InstallEnvPage(QWidget *parent)
    : QWizardPage(parent),
      d_ptr(new InstallEnvPagePrivate(this))
{
    Q_D(InstallEnvPage);
    d->setupUi();
    registerField("installFinished*", this);
    connect(&d->m_subProcess, &SubProcess::output, [=](QString msg) {
        if (msg.length() >2 && msg.at(0) == QChar('\r') && msg.at(1) != QChar('\n'))
        {
            // TODO：碰到回车符，认为是回到行首
            // return ;
        }

        msg.replace("[1;31m", "");
        msg.replace("[1;33m", "");
        msg.replace("[0m", "");
        d->m_output->appendText(msg);
        if (d->m_highlighter == nullptr) {
            d->m_highlighter = new KeywordHighlighter(d->m_output->document());
        }
    });

    connect(&d->m_subProcess, &SubProcess::finished, [=](bool result) {
        setField("installFinished", true);  // 设置字段值
        emit completeChanged();
        d->m_installationSuccess = result;

        // 在这里添加安装完成后的逻辑
           if (result) {
               qDebug() << "安装成功！";
               // 例如：弹出提示、跳转页面等
           } else {
               qDebug() << "安装失败！";
               MessageBoxUtils::QMESSAGE_BOX_WARNING(this, QObject::tr("Warning"),
                                                     tr("Install Failed"),
                                                     QObject::tr("OK"));
           }
    });
}

InstallEnvPage::~InstallEnvPage()
{
}

void InstallEnvPage::initializePage()
{
    bool needBaseInstall = !wizard()->field("isBaseInstalled").toBool();
    bool needRuntimeInstall = !wizard()->field("isRuntimeInstalled").toBool();
    QString baseStr = field("base").toString();
    QString runtimeStr = field("runtime").toString();
    if (needBaseInstall) {
        qDebug() << "需要安装基础环境";

    }
    if (needRuntimeInstall) {
        qDebug() << "需要安装Runtime环境";
    }

    startInstall(needBaseInstall ? baseStr : "", needRuntimeInstall ? runtimeStr : "");

}

void InstallEnvPage::startInstall(QString baseStr, QString runtimeStr)
{
    Q_D(InstallEnvPage);
    d->m_output->clear();
    QStringList args;
    args << "/opt/kaiming-tools/bin/kaiming";
    args << "install";
    args << "-y";
    if(!baseStr.isEmpty())
    {
        KMRef ref1;
        ref1.parseFromInfo(baseStr);
        d->m_baseRef = ref1.m_id + "/" + ref1.m_version + "/" + "devel" + "/" + ref1.m_channel;
        qDebug()<< "base:" << d->m_baseRef;
        args << d->m_baseRef;
    }
    if(!runtimeStr.isEmpty())
    {
        KMRef ref2;
        ref2.parseFromInfo(runtimeStr);
        d->m_runtimeRef = ref2.m_id + "/" + ref2.m_version + "/" + "devel" + "/" + ref2.m_channel;
        qDebug() << "runtime：" << d->m_runtimeRef;
        args << d->m_runtimeRef;
    }

    d->m_subProcess.start("pkexec", args);
}

bool InstallEnvPage::isComplete() const
{
    return field("installFinished").toBool();
}

bool InstallEnvPage::validatePage()
{
    Q_D(InstallEnvPage);
    QString base = field("base").toString();
    QString runtime = field("runtime").toString();
    KMRef ref1;
    ref1.parseFromInfo(base);
    d->m_baseRef = ref1.m_id + "/" + ref1.m_version + "/" + "devel" + "/" + ref1.m_channel;

    KMRef ref2;
    ref2.parseFromInfo(runtime);
    d->m_runtimeRef = ref2.m_id + "/" + ref2.m_version + "/" + "devel" + "/" + ref2.m_channel;

    // 检查基础环境
    if (!wizard()->field("isBaseInstalled").toBool()) {
        if (!KMBuilderSearch::isRefFromLocal("base", base)) {
            MessageBoxUtils::QMESSAGE_BOX_WARNING(this, QObject::tr("Warning"),
                                                  tr("The basic environment was not installed successfully."
                                                     " Please use the command to manually install it: sudo kaiming install %1")
                                                  .arg(d->m_baseRef),
                                                  QObject::tr("OK"));
            return false;
        }
    }

    // 检查运行时环境
    if (!wizard()->field("isRuntimeInstalled").toBool() && !runtime.isEmpty()) {
        if (!KMBuilderSearch::isRefFromLocal("runtime", runtime)) {
            MessageBoxUtils::QMESSAGE_BOX_WARNING(this, QObject::tr("Warning"),
                                                  tr("The runtime environment was not installed successfully."
                                                     " Please use the command to manually install it: sudo kaiming install %1")
                                                  .arg(d->m_runtimeRef),
                                                  QObject::tr("OK"));
            return false;
        }
    }
    return true;
}

InstallEnvPagePrivate::InstallEnvPagePrivate(InstallEnvPage * parent)
    : QObject(parent),
      q_ptr(parent),
      m_subProcess(this)
{}

InstallEnvPagePrivate::~InstallEnvPagePrivate()
{}

void InstallEnvPagePrivate::setupUi()
{
    Q_Q(InstallEnvPage);

    // 创建自定义标题控件
    QLabel *titleLabel = new QLabel(tr("Environment Installation"));
    titleLabel->setStyleSheet("font-size: 16px; font-weight: bold;");
    QLabel *subTitleLabel = new QLabel(tr("Installing required dependencies..."));
    subTitleLabel->setStyleSheet("font-size: 12px; color: #666;");

    // 日志显示区域
    m_output = new OutputEdit;
    {
        m_output->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        m_output->setReadOnly(true);
    }

    // 布局管理
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(titleLabel);
    mainLayout->addSpacing(10);
    mainLayout->addWidget(subTitleLabel);
    mainLayout->addSpacing(10);
    mainLayout->addWidget(new QLabel(tr("Installation Log:")));
    mainLayout->addWidget(m_output);
    q->setLayout(mainLayout);
}

#include "installenvpage.moc"
#include "moc_installenvpage.cpp"

